#!/usr/bin/env python
# -*- coding: utf-8 -*-

# Copyright 2010, 2011 Richard Dymond (rjdymond@gmail.com)
#
# This file is part of SkoolKit.
#
# SkoolKit is free software: you can redistribute it and/or modify it under the
# terms of the GNU General Public License as published by the Free Software
# Foundation, either version 3 of the License, or (at your option) any later
# version.
#
# SkoolKit is distributed in the hope that it will be useful, but WITHOUT ANY
# WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
# A PARTICULAR PURPOSE. See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along with
# SkoolKit. If not, see <http://www.gnu.org/licenses/>.

import sys
import os.path
from skoolkit.skoolparser import SkoolParser
from skoolkit.skoolctl import CtlWriter, BLOCKS, BLOCK_TITLES, BLOCK_DESC, REGISTERS, BLOCK_COMMENTS, SUBBLOCKS, COMMENTS

def parse_args(args):
    elements = BLOCKS + BLOCK_TITLES + BLOCK_DESC + REGISTERS + BLOCK_COMMENTS + SUBBLOCKS + COMMENTS
    write_hex = False
    files = []
    i = 0
    while i < len(args):
        arg = args[i]
        if arg[:2] == '-w':
            elements = arg[2:]
        elif arg == '-h':
            write_hex = True
        elif arg[0] == '-':
            print_usage()
        else:
            files.append(arg)
        i += 1

    if len(files) != 1:
        print_usage()
    return files[0], elements, write_hex

def print_usage():
    sys.stderr.write('Usage: %s [options] <skoolfile>\n' % os.path.basename(sys.argv[0]))
    sys.stderr.write('\nOptions:\n')
    sys.stderr.write("  -wX  Write only these elements, where X is one or more of:\n")
    sys.stderr.write("         %s = block types and addresses\n" % BLOCKS)
    sys.stderr.write("         %s = block titles\n" % BLOCK_TITLES)
    sys.stderr.write("         %s = block descriptions\n" % BLOCK_DESC)
    sys.stderr.write("         %s = registers\n" % REGISTERS)
    sys.stderr.write("         %s = mid-block comments and block end comments\n" % BLOCK_COMMENTS)
    sys.stderr.write("         %s = sub-block types and addresses\n" % SUBBLOCKS)
    sys.stderr.write("         %s = instruction-level comments\n" % COMMENTS)
    sys.stderr.write("  -h   Write addresses in hexadecimal format\n")
    sys.exit(1)

def run(skoolfile, elements, write_hex):
    parser = SkoolParser(skoolfile)
    writer = CtlWriter(parser.memory_map, elements, write_hex)
    writer.write()

def main():
    run(*parse_args(sys.argv[1:]))

if __name__ == '__main__':
    main()
